import { parseJsonString } from "@egg/hooks"
import { z } from "zod"
import { zodToJsonSchema } from "zod-to-json-schema"

import { Context } from "../../types"
import llm from "../../utils/llm"
import { cleanLLMRes } from "../../utils/llm/base"

/**
 * 基础意图模式
 */
const BaseIntentSchema = z.object({
  intent: z.number().min(1).max(13),
})

/**
 * 简报模式
 */
const BriefingSchema = BaseIntentSchema.extend({
  intent: z.literal(5),
  link: z.string().url(),
  userDescription: z.string().min(1),
})

const BriefingLinkSchema = z.object({
  intent: z.literal(14),
  link: z.string().url(),
})

/**
 * 通用响应模式
 */
const CommonResponseSchema = BaseIntentSchema.extend({
  intent: z.literal(2),
  message: z.string().min(1),
})

/**
 * 意图模式
 */
const IntentSchema = z.union([
  BriefingSchema,
  CommonResponseSchema,
  BaseIntentSchema,
  BriefingLinkSchema,
])

type BaseIntent = z.infer<typeof BaseIntentSchema>
type Briefing = z.infer<typeof BriefingSchema>
type BriefingLink = z.infer<typeof BriefingLinkSchema>
type CommonResponse = z.infer<typeof CommonResponseSchema>
export type Intent = z.infer<typeof IntentSchema>

/**
 * JSON模式
 */
const jsonSchema = zodToJsonSchema(IntentSchema)

/**
 * 代理函数
 * @param {Context} ctx - 上下文对象
 * @returns {Promise<Intent>} - 返回意图对象
 */
const agent = async (ctx: Context): Promise<Intent> => {
  const {
    larkBody: { msgText },
    logger,
  } = ctx

  let attempts = 0
  while (attempts < 3) {
    const res = await llm.invoke(
      "intentRecognitionNext",
      {
        userInput: msgText,
        time: new Date().toLocaleString("zh-CN", { timeZone: "Asia/Shanghai" }),
        jsonSchema,
      },
      "test",
      0,
      true
    )

    const rawIntent = cleanLLMRes(res as string)
    const parsedIntent = parseJsonString(rawIntent, null)

    if (parsedIntent) {
      try {
        IntentSchema.parse(parsedIntent)
        logger.debug("Intent is valid: " + JSON.stringify(parsedIntent))
        return parsedIntent
      } catch (e: any) {
        logger.error("Invalid intent: " + String(e.errors))
      }
    } else {
      logger.error("Parsed intent is null")
    }

    attempts++
  }

  return {
    intent: 1,
  }
}

const isBaseIntent = (intent: Intent): intent is BaseIntent => {
  return intent.intent !== 5 && intent.intent !== 2
}

const isBriefing = (intent: Intent): intent is Briefing => {
  return intent.intent === 5
}

const isBriefingLink = (intent: Intent): intent is BriefingLink => {
  return intent.intent === 14
}

const isCommonResponse = (intent: Intent): intent is CommonResponse => {
  return intent.intent === 2
}

/**
 * 意图代理对象
 */
const intentAgent = {
  agent,
  isBaseIntent,
  isBriefing,
  isCommonResponse,
  isBriefingLink,
}

export default intentAgent
